//==============================================================================
/*	Objetivo:	Simulador normalizado de engranajes rectos
			    Creacion mediante opengl de los mecanismos usados en la
			    simulacion.


Copyright (C) 2011 Daniel Costas Rodriguez

Este programa es software libre: usted puede redistribuirlo y/o modificarlo 
bajo los trminos de la Licencia Pblica General GNU publicada por la Fundacin
para el Software Libre, ya sea la versin 3 de la Licencia, o (a su eleccin)
cualquier versin posterior.

Este programa se distribuye con la esperanza de que sea til, pero SIN GARANTIA
ALGUNA; ni siquiera la garanta implcita MERCANTIL o de APTITUD PARA UN
PROPSITO DETERMINADO. Consulte los detalles de la Licencia Pblica General
GNU para obtener una informacion ms detallada.

Debera haber recibido una copia de la Licencia Pblica General GNU junto a 
este programa.
*/

//==============================================================================

#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include <math.h>
#include "mecanismos.h"		// Incluido por las definiciones de los perfiles



//------------------------------------------------------------------------------
//	Nombre:		Cilindro
//	Funcion:	Dibuja el cilindro base del engranaje dado el radio R
//------------------------------------------------------------------------------

void Cilindro(float R, float *color) {

    GLUquadricObj *Base;
    Base = gluNewQuadric();

    glPushMatrix();
    glColor3f(color[0], color[1], color[2]);
    gluDisk(Base, R * .1, R, 60, 2);
    glPopMatrix();

}
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
//	Nombre:		Circunferencia
//	Funcion:	Dibuja una circunferencia dado el radio R y el color rgb
//------------------------------------------------------------------------------

void Circunferencia(float R, float *color) {

    int numero_puntos = 300;
    float angulo;
    glColor3f(color[0], color[1], color[2]);
    
	glBegin(GL_LINE_LOOP);
	for (int i = 0; i < numero_puntos; i++) {

        angulo = 2 * M_PI * i / numero_puntos;
        glVertex2f(R * cos(angulo), R * sin(angulo));

    }
    glEnd();
}
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
//	Nombre:		Cremallera
//	Funcion:	Dibuja una cremallera
//------------------------------------------------------------------------------

void Cremallera(float *color, float *dato) {


    // Paso del angulo de presion de grados a radianes
    float angulo = dato[5] * M_PI / 180;
    float angulo_2 = dato[10] * M_PI / 180;

    glPushMatrix();

    glColor3f(color[0], color[1], color[2]);

    glTranslatef((GLfloat) dato[0], (GLfloat) dato[1], 0);
    glRotatef((GLfloat) dato[2], 0, 0, 1);
    
    
	if (dato[9]) {

        // El diente de la cremallera tiene una deformacion circular
        // con flecha dada por dato[9]

        // Radio de la circunferencia distorsion del perfil estandar
        float radio = pow(dato[3], 2) / 8 / dato[9] / pow(cos(angulo), 2) + 
			dato[9] / 2;


        // Dientes
        for (int i = -2; i < 3; i++) {

            glBegin(GL_POLYGON);
            
			// primer vertice
            glVertex2f(-dato[4] / 4 - dato[3] / 2 * tan(angulo) + 
					i * dato[4], dato[6]);

            // vertices de la linea curva
            float beta = M_PI - acos( (dato[9] - radio) / radio );
            float alfa_1 = M_PI - angulo + beta;
            float alfa_2 = M_PI - angulo - beta;
            float alfa = (alfa_1 - alfa_2) / 10;

            // Centro de la circunferencia
            float CirX = -dato[4] / 4 - (dato[9] - radio) * cos(angulo) +
                    i * dato[4];
            float CirY = dato[3] / 2 + (dato[9] - radio) * sin(angulo);
            
			// Trazado de los puntos de la circunferencia
            for (float j = alfa_1; j > alfa_2; j -= alfa) {

                glVertex2f(
						CirX + radio * cos(j), 
						CirY + radio * sin(j) + dato[6]
						);
            }

            // segundo vertice
            glVertex2f(
					-dato[4] / 4 + dato[3] / 2 * tan(angulo) + i * dato[4],
                    dato[3] + dato[6]
					);

            // tercer vertice
            glVertex2f(
					dato[4] / 4 - dato[3] / 2 * tan(angulo) + i * dato[4],
                    dato[3] + dato[6]
					);

            // vertices de la linea curva
            alfa_1 = angulo + beta;
            alfa_2 = angulo - beta;
            alfa = (alfa_1 - alfa_2) / 10;

            // Nuevas coordenadas del centro para el lado simetrico del diente
            CirX = dato[4] / 4 + (dato[9] - radio) * cos(angulo) + i * dato[4];
            CirY = dato[3] / 2 + (dato[9] - radio) * sin(angulo);

            for (float j = alfa_1; j > alfa_2; j -= alfa) {

                glVertex2f(
						CirX + radio * cos(j),
                        CirY + radio * sin(j) + dato[6]
						);

            }

            // cuarto vertice
            glVertex2f(
					dato[4] / 4 + dato[3] / 2 * tan(angulo) + i * dato[4],
                    dato[6]
					);

            glEnd();

        }

    } else {

        if (dato[7]) {
        
			// Diente recomendado por ISO 6336

			if (dato[8]) {

				// Diente con cabeza redondeada y con undercut

	            // los Dientes
	            for (int i = -2; i < 3; i++) {
					
					float V1x, V1y, V2x, V2y, delta_i, delta_d;
					
					// Centro del radio de cabeza del lado izdo del diente
					float Ei_x = -dato[4] / 4 + dato[3] / 2 * tan(angulo) +
						dato[7] * ( (1 - sin(angulo)) / cos(angulo)) +
						i * dato[4];
					float Ei_y = dato[3] - dato[7] + dato[6];

					// Centro del radio de cabeza del lado derecho del diente
					float Ed_x = dato[4] / 4 - dato[3] / 2 * tan(angulo_2) -
						dato[7] * ( (1 - sin(angulo_2)) / cos(angulo_2)) +
						i * dato[4];
					float Ed_y = dato[3] - dato[7] + dato[6];
				
					// Vertices del lateral izquierdo
					float Ax = - dato[4] / 4 - dato[3] / 2 * tan(angulo) +
						dato[8] / cos(angulo) + i * dato[4];
					float Ay = dato[6];

					float limiteIz = dato[7] * (1 - cos(angulo));
					if ( dato[8] > limiteIz ) {

						V1x = - dato[4] / 4 + dato[3] / 2 * tan(angulo) + 
							dato[7] * ( (1 - sin(angulo) - 
							cos(angulo)) / cos(angulo)) +
							i * dato[4];
						V1y = dato[3] + dato[7] * ( (1 - sin(angulo) - 
									cos(angulo)) / sin(angulo) ) - 
							dato[8] / sin(angulo) + dato[6];
						if (V1y < 0) {
							V1y = 0;
						}
						delta_i = 0;

					} else {

						delta_i = angulo - angulo / limiteIz * dato[8];
						V1x = Ei_x - dato[7] * cos(delta_i) ;
						V1y = Ei_y + dato[7] * sin(delta_i) ;

					}

					// Vertices del lateral derecho
					float Axx = dato[4] / 4 + dato[3] / 2 * tan(angulo_2) - 
						dato[8] / cos(angulo_2) + i * dato[4];
					float Ayy = dato[6];

					float limiteDr = dato[7] * (1 - cos(angulo_2));
					if (dato[8] > limiteDr) {

						V2x = dato[4] / 4 - dato[3] / 2 * tan(angulo_2) -
							dato[7] * ( (1 - sin(angulo_2) - 
										cos(angulo_2)) / cos(angulo_2)) +
							i * dato[4];
						V2y = dato[3] + dato[7] * ( (1 - sin(angulo_2) - 
									cos(angulo_2)) / sin(angulo_2)) - 
							dato[8] / sin(angulo_2) + dato[6];
						if (V2y < 0) {
							V2y = 0;
						}
						delta_d = 0;

					} else {

						delta_d = angulo_2 - angulo_2 / limiteDr * dato[8];
						V2x = Ed_x + dato[7] * cos(angulo_2) ;
						V2y = Ed_y + dato[7] * sin(angulo_2) ;

					}

					// BASE UNDERCUT DEL DIENTE
	                glBegin(GL_POLYGON);
					glVertex2f(Ax, Ay);
					glVertex2f(V1x, V1y);
					glVertex2f(V2x, V2y);
					glVertex2f(Axx, Ayy);
					glEnd();

		
					// CABEZA REDONDEADA DEL DIENTE
					glBegin(GL_POLYGON);
					glVertex2f(V1x, V1y);
					
					// angulos de comienzo y fin del arco del lado izquierdo
					float alfa_1 = delta_i;
					float alfa_2 = M_PI / 2;
					float alfa = (alfa_2 - alfa_1) / 6;

					// Arco de cabeza. Lado izquierdo
					for (float j = alfa_1; j < alfa_2; j += alfa) {

						float Px = Ei_x - dato[7] * cos(j);
						float Py = Ei_y + dato[7] * sin(j);
						glVertex2f(Px, Py);
					
					}

					// angulos de comienzo y fin del arco derecho 
					alfa_2 = M_PI - delta_d;
					alfa_1 = M_PI / 2;
					alfa = (alfa_2 - alfa_1) / 6;
					
					// Arco de cabeza. Lado derecho
					for (float j = alfa_1; j < alfa_2; j += alfa) {

						float Px = Ed_x - dato[7] * cos(j);
						float Py = Ed_y + dato[7] * sin(j);
						glVertex2f(Px, Py);
						
					}

					glVertex2f(V2x, V2y);
					glEnd();

				}

			} else {

				// Solo existe redondeo de la cabeza del diente de la cremallera
				// sin undercut, se construye el diente como un bloque unico
				
	            // Dientes
	            for (int i = -2; i < 3; i++) {
	
	                glBegin(GL_POLYGON);
	
	                // LADO IZQUIERDO DEL DIENTE DE LA CREMALLERA
	
	                // Vertice 1
	                float Ax = -dato[4] / 4 - dato[3] / 2 * tan(angulo) +
	                        i * dato[4];
	                float Ay = dato[6];
	                glVertex2f(Ax, Ay);
	
	                // Coordenadas del centro del radio de cabeza
	                float Ex = -dato[4] / 4 + dato[3] / 2 * tan(angulo) + 
						dato[7] * ((1 - sin(angulo)) / cos(angulo)) + 
						i * dato[4];
	                float Ey = dato[3] - dato[7] + dato[6];
	
					if (angulo > 0) {

		                // Vertice 2
		                float V2x = Ex - dato[7];
		                float V2y = dato[3] + dato[7] * ((1 - sin(angulo) -
	                        cos(angulo)) / sin(angulo)) + dato[6];
		                glVertex2f(V2x, V2y);

					}
	
	                // angulo donde empieza el arco de circunferencia de cabeza
	                // y angulo donde termina
	                float alfa_1 = angulo;
	                float alfa_2 = M_PI / 2;
	                float alfa = (alfa_2 - alfa_1) / 6;
	
	                // Arco de cabeza
	                for (float j = alfa_1; j < alfa_2; j += alfa) {

	                    float Px = Ex - dato[7] * cos(j);
	                    float Py = Ey + dato[7] * sin(j);
	                    glVertex2f(Px, Py);

	                }
	
	                // LADO DERECHO DEL DIENTE DE LA CREMALLERA
	
	                // Coordenadas del centro del radio de cabeza
	                Ex = dato[4] / 4 - dato[3] / 2 * tan(angulo_2) - 
						dato[7] * ((1 - sin(angulo_2)) / cos(angulo_2)) + 
						i * dato[4];
	
	                // Arco de cabeza
					alfa_2 = M_PI - angulo_2;
	                alfa_1 = M_PI / 2;
					alfa = (alfa_2 - alfa_1) / 6;
	                for (float j = alfa_1; j < alfa_2; j += alfa) {

	                    float Px = Ex - dato[7] * cos(j);
	                    float Py = Ey + dato[7] * sin(j);
	                    glVertex2f(Px, Py);

	                }

					if (angulo_2 > 0) {

		                // Vertice 2'
		                float V2x = Ex + dato[7];
		                float V2y = dato[3] + dato[7] * ((1 - sin(angulo_2) -
	                        cos(angulo_2)) / sin(angulo_2)) + dato[6];
		                glVertex2f(V2x, V2y);

					}
	
	                // Vertice 1'
	                float Dx = dato[4] / 4 + dato[3] / 2 * tan(angulo_2) +
	                        i * dato[4];
	                float Dy = dato[6];
	                glVertex2f(Dx, Dy);
	
	
	                glEnd();

				}

            }


        } else {

            // Diente estandar
            
            // Dientes
            for (int i = -2; i < 3; i++) {

                glBegin(GL_POLYGON);
                glVertex2f(
						-dato[4] / 4 - dato[3] / 2 * tan(angulo) + i * dato[4],
						dato[6]
						);
                glVertex2f(
						-dato[4] / 4 + dato[3] / 2 * tan(angulo) + i * dato[4],
						dato[3] + dato[6]
						);
                glVertex2f(
						dato[4] / 4 - dato[3] / 2 * tan(angulo_2) + i * dato[4],
						dato[3] + dato[6]
						);
                glVertex2f(
						dato[4] / 4 + dato[3] / 2 * tan(angulo_2) + i * dato[4],
						dato[6]
						);
                glEnd();

            }

        }

    }

    if (dato[6] > 0) {

        // Existe penetracion y el diente se desplaza con respecto a su eje de 
		// giro por lo que se crea una base al diente conectada al eje de giro

        for (int i = -2; i < 3; i++) {

            glBegin(GL_POLYGON);
            glVertex2f(
					-dato[4] / 4 - dato[3] / 2 * tan(angulo) - 
					dato[6] * tan(angulo) + dato[8] / cos(angulo) + i * dato[4],
					0
					);
            glVertex2f(
					-dato[4] / 4 - dato[3] / 2 * tan(angulo) + 
					dato[8] / cos(angulo) + i * dato[4],
                    dato[6]
					);
            glVertex2f(
					dato[4] / 4 + dato[3] / 2 * tan(angulo_2) - 
					dato[8] / cos(angulo_2) + i * dato[4],
                    dato[6]
					);
            glVertex2f(
					dato[4] / 4 + dato[3] / 2 * tan(angulo_2) +
					dato[6] * tan(angulo_2) - 
					dato[8] / cos(angulo_2) + i * dato[4],
					0
					);
            glEnd();

        }

    }


    // Base. Eje de rotacion.
    glBegin(GL_QUADS);
    glVertex2f(-3 * dato[4], 0);
    glVertex2f(3 * dato[4], 0);
    glVertex2f(3 * dato[4], -dato[3] / 2);
    glVertex2f(-3 * dato[4], -dato[3] / 2);
    glEnd();

    glPopMatrix();

}
//------------------------------------------------------------------------------
